package com.zanxiang.sdk.common.resolver;

import com.zanxiang.common.enums.HttpStatusEnum;
import com.zanxiang.common.exception.CustomException;
import com.zanxiang.common.utils.StringUtils;
import com.zanxiang.sdk.common.annotation.ValidLogin;
import com.zanxiang.sdk.common.util.DeviceCheckUtil;
import com.zanxiang.sdk.domain.dto.UserTokenDTO;
import com.zanxiang.sdk.domain.params.UserData;
import com.zanxiang.sdk.service.Impl.pay.AlipayServiceImpl;
import com.zanxiang.sdk.service.UserTokenService;
import lombok.extern.slf4j.Slf4j;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.core.MethodParameter;
import org.springframework.stereotype.Component;
import org.springframework.stereotype.Service;
import org.springframework.web.bind.support.WebDataBinderFactory;
import org.springframework.web.context.request.NativeWebRequest;
import org.springframework.web.method.support.HandlerMethodArgumentResolver;
import org.springframework.web.method.support.ModelAndViewContainer;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;

/**
 * token转用户信息解析器
 *
 * @author xufeng
 * @date 2022/6/20 09:59
 */
@Component
@Service
@Slf4j
public class TokenArgumentResolver implements HandlerMethodArgumentResolver {

    private static final Logger logger = LoggerFactory.getLogger(AlipayServiceImpl.class);

    @Resource
    private UserTokenService userTokenService;

    @Override
    public boolean supportsParameter(MethodParameter parameter) {
        return parameter.getParameterType().equals(UserData.class);
    }

    @Override
    public Object resolveArgument(MethodParameter parameter, ModelAndViewContainer mavContainer, NativeWebRequest webRequest, WebDataBinderFactory binderFactory) throws Exception {
        HttpServletRequest request = webRequest.getNativeRequest(HttpServletRequest.class);
        if (request == null) {
            throw new CustomException(HttpStatusEnum.UNKNOWN_ERROR);
        }
        //是否强制验证登录状态
        boolean b = parameter.hasParameterAnnotation(ValidLogin.class);
        String token = request.getHeader("token");
        if (b && StringUtils.isEmpty(token)) {
            throw new CustomException(HttpStatusEnum.USER_NO_LOGIN);
        }
        String userAgent = request.getHeader("user-agent");
        logger.info("user-agent:{}", userAgent);
        Integer deviceType = DeviceCheckUtil.getType(userAgent);
        UserData data = new UserData();
        data.setDeviceType(deviceType);
        data.setDeviceSystem(DeviceCheckUtil.getDeviceSystem(userAgent));
        logger.info("UserData:{}", data);
        if (StringUtils.isEmpty(token)) {
            if (b) {
                throw new CustomException(HttpStatusEnum.USER_NO_LOGIN);
            }
            return data;
        }
        UserTokenDTO tokenInfoByTokenDevice = userTokenService.getTokenInfoByTokenDevice(token, deviceType);
        if (tokenInfoByTokenDevice == null) {
            if (b) {
                throw new CustomException(HttpStatusEnum.USER_NO_LOGIN);
            }
            return data;
        }
        if (b && tokenInfoByTokenDevice.getUserId() == null) {
            throw new CustomException(HttpStatusEnum.USER_NO_LOGIN);
        }
        data.setUserId(tokenInfoByTokenDevice.getUserId());
        return data;
    }
}
